@@ -122,6 +122,46 @@ char parse_bool(char *str)
     return 0;
 }
 
+int parse_ycbcr_matrix(char *str)
+{
+    while (*str == ' ' || *str == '\t')
+        str++;
+    if (*str == '\0')
+        return YCBCR_DEFAULT;
+
+    char *end = str + strlen(str);
+    while (end[-1] == ' ' || end[-1] == '\t')
+        end--;
+
+    // Trim a local copy of the input that we know is safe to
+    // modify. The buffer is larger than any valid string + NUL,
+    // so we can simply chop off the rest of the input.
+    char buffer[16];
+    size_t n = FFMIN(end - str, sizeof buffer - 1);
+    strncpy(buffer, str, n);
+    buffer[n] = '\0';
+
+    if (!strcasecmp(buffer, "none"))
+        return YCBCR_NONE;
+    if (!strcasecmp(buffer, "tv.601"))
+        return YCBCR_BT601_TV;
+    if (!strcasecmp(buffer, "pc.601"))
+        return YCBCR_BT601_PC;
+    if (!strcasecmp(buffer, "tv.709"))
+        return YCBCR_BT709_TV;
+    if (!strcasecmp(buffer, "pc.709"))
+        return YCBCR_BT709_PC;
+    if (!strcasecmp(buffer, "tv.240m"))
+        return YCBCR_SMPTE240M_TV;
+    if (!strcasecmp(buffer, "pc.240m"))
+        return YCBCR_SMPTE240M_PC;
+    if (!strcasecmp(buffer, "tv.fcc"))
+        return YCBCR_FCC_TV;
+    if (!strcasecmp(buffer, "pc.fcc"))
+        return YCBCR_FCC_PC;
+    return YCBCR_UNKNOWN;
+}
+
 void ass_msg(ASS_Library *priv, int lvl, char *fmt, ...)
 {
     va_list va;
@@ -165,14 +205,20 @@ unsigned ass_utf8_get_char(char **str)
  * \param track track
  * \param name style name
  * \return index in track->styles
- * Returnes 0 if no styles found => expects at least 1 style.
- * Parsing code always adds "Default" style in the end.
+ * Returns 0 if no styles found => expects at least 1 style.
+ * Parsing code always adds "Default" style in the beginning.
  */
 int lookup_style(ASS_Track *track, char *name)
 {
     int i;
-    if (*name == '*')
-        ++name;                 // FIXME: what does '*' really mean ?
+    // '*' seem to mean literally nothing;
+    // VSFilter removes them as soon as it can
+    while (*name == '*')
+        ++name;
+    // VSFilter then normalizes the case of "Default"
+    // (only in contexts where this function is called)
+    if (strcasecmp(name, "Default") == 0)
+        name = "Default";
     for (i = track->n_styles - 1; i >= 0; --i) {
         if (strcmp(track->styles[i].Name, name) == 0)
             return i;
@@ -181,7 +227,27 @@ int lookup_style(ASS_Track *track, char *name)
     ass_msg(track->library, MSGL_WARN,
             "[%p]: Warning: no style named '%s' found, using '%s'",
             track, name, track->styles[i].Name);
-    return i;                   // use the first style
+    return i;
+}
+
+/**
+ * \brief find style by name as in \r
+ * \param track track
+ * \param name style name
+ * \return style in track->styles
+ * Returns NULL if no style has the given name.
+ */
+ASS_Style *lookup_style_strict(ASS_Track *track, char *name)
+{
+    int i;
+    for (i = track->n_styles - 1; i >= 0; --i) {
+        if (strcmp(track->styles[i].Name, name) == 0)
+            return track->styles + i;
+    }
+    ass_msg(track->library, MSGL_WARN,
+            "[%p]: Warning: no style named '%s' found",
+            track, name);
+    return NULL;
 }
 
 #ifdef CONFIG_ENCA
